# How to Contribute

Help with DBI is always welcome in all areas: code, tests, documentation.
Just reach out to se where we think the pressure point are.

## Issues

Bugs and problems can be reported at:

https://github.com/perl5-dbi/dbi/issues

## Questions

Questions about usage can be answered in a number of places:

### IRC

You may find helpful people at #dbi on irc.perl.org - people might not be
around the moment you ask the question, but an answer is likely to be given.

## Testing

You can help by testing DBI as widely as possible. Running the tests
on a wide variety of platforms in as many configurations as possible (32bit
vs 64-bit, threaded vs non-threaded, big-endian vs lower endian, quadmath,
parallel testing vs single process testing, limited resources vs extreme
resources), database types and versions, DBI versions, and other factors is
a great help in uncovering problems and improving DBI.

## Code development

Development happens in a git repo. The canonical location is currently
at https://github.com/perl5-dbi/dbi/

Create your own copy of the repo by running:

git clone https://github.com/perl5-dbi/dbi.git

## Guides

### Code style and layout

This code has long history and was developed by multiple authors. Try to
keep the style of the new or modified code in line with the surrounding code.
