This is the Security Policy for Perl DBI.

Report security issues via [GitHub Security Advisories](https://github.com/perl5-dbi/dbi/security/advisories).

The maintainers will respond, and release fixes as soon as possible.
However, this project is maintained by volunteers, and they cannot guarantee a rapid response.

If the issue is urgent, then you can also notify the CPAN Security Group (CPANSec) via email to <cpan-security@security.metacpan.org>.

Please do not report security problems on public forums or in repository issues.

Only the latest release of DBI will be supported.

This policy was updated on 11 August 2026.

If this policy or the release is more than two years old, then you should check for a more recent version of [DBI on CPAN](https://metacpan.org/dist/DBI) or the main branch of the [DBI git repository](https://github.com/perl5-dbi/dbi).
