# Id() requires a character vector

    Code
      Id(1)
    Condition
      Error:
      ! All elements of `...` must be strings.

# has a decent print method

    Code
      Id("a", "b")
    Output
      <Id> "a"."b"

