% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dbListConnections.R
\name{dbListConnections}
\alias{dbListConnections}
\title{List currently open connections}
\usage{
dbListConnections(drv, ...)
}
\arguments{
\item{drv}{A object inheriting from \linkS4class{DBIDriver}}

\item{...}{Other arguments passed on to methods.}
}
\value{
a list
}
\description{
DEPRECATED, drivers are no longer required to implement this method.
Keep track of the connections you opened if you require a list.
}
\seealso{
Other DBIDriver generics: 
\code{\link{DBIDriver-class}},
\code{\link{dbCanConnect}()},
\code{\link{dbConnect}()},
\code{\link{dbDataType}()},
\code{\link{dbDriver}()},
\code{\link{dbGetInfo}()},
\code{\link{dbIsReadOnly}()},
\code{\link{dbIsValid}()}
}
\concept{DBIDriver generics}
\keyword{internal}
