=head1 NAME

TASKS - Want to help? These things need doing...

=head2 Increase test coverage

More tests need to be added to test the codes that not urrently being tested.

It's pretty poor right now:

   http://pjcj.sytes.net/cover/latest/DBI-1.52/coverage.html

Start with improving the subroutine coverage

   http://pjcj.sytes.net/cover/latest/DBI-1.52/blib-lib-DBI-pm--subroutine.html

=head2 Test the proxy

The current t/80proxy.t is isolated from the rest of the test suite so actually
tests very little, and what it does test is duplicating other tests.

Ideally the proxy should be tested in the same way as DBI::PurePerl.  In other
words, by creating wrappers test files for each test file that set
$ENV{DBI_AUTOPROXY} and run the original test. They'll also need to start and
stop a proxy server.

=head2 Fixing bugs

The official bug list is here:

   http://rt.cpan.org/NoAuth/Bugs.html?Dist=DBI

Naturally I'll offer direction and guidance on any you want to tackle.
I've also got a few that could be entered into rt.cpan.org.

=head2 Others

General:

    Protect trace_msg from SIGPIPE?
    prepare(...,{ Err=>\my $isolated_err, ...})
    Add trace module that just records the last N trace messages into an array
        and prepends them to any error message to provide context for the error.
    Document DBI_PROFILE_FLOCK and LockFile attrib in DBI::ProfileData and DBI::ProfileDumper

Performance:

    Move _new_sth to DBI::db::_new_sth (leave alias) and implement in C
        Or call _new_child and move to DBI::common?

    Implement FETCH_many() in C

    Add high-res dbi_time for windows - via Time::HiRes glob replace dbi_time()?

=cut
