% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02-DBIDriver.R
\docType{class}
\name{DBIDriver-class}
\alias{DBIDriver-class}
\title{DBIDriver class}
\description{
Base class for all DBMS drivers (e.g., RSQLite, MySQL, PostgreSQL).
The virtual class \code{DBIDriver} defines the operations for creating
connections and defining data type mappings.  Actual driver classes, for
instance \code{RPostgres}, \code{RMariaDB}, etc. implement these operations in a
DBMS-specific manner.
}
\seealso{
Other DBI classes: 
\code{\link{DBIConnection-class}},
\code{\link{DBIConnector-class}},
\code{\link{DBIObject-class}},
\code{\link{DBIResult-class}},
\code{\link{DBIResultArrow-class}}

Other DBIDriver generics: 
\code{\link{dbCanConnect}()},
\code{\link{dbConnect}()},
\code{\link{dbDataType}()},
\code{\link{dbDriver}()},
\code{\link{dbGetInfo}()},
\code{\link{dbIsReadOnly}()},
\code{\link{dbIsValid}()},
\code{\link{dbListConnections}()}
}
\concept{DBI classes}
\concept{DBIDriver generics}
