% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{dbGetDBIVersion}
\alias{dbGetDBIVersion}
\title{Determine the current version of the package.}
\usage{
dbGetDBIVersion()
}
\description{
Determine the current version of the package.
}
\keyword{internal}
