% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\docType{data}
\name{.SQL92Keywords}
\alias{.SQL92Keywords}
\title{Keywords according to the SQL-92 standard}
\format{
An object of class \code{character} of length 220.
}
\usage{
.SQL92Keywords
}
\description{
A character vector of SQL-92 keywords, uppercase.
}
\examples{
"SELECT" \%in\% .SQL92Keywords
}
\keyword{datasets}
