% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dbSetDataMappings.R
\name{dbSetDataMappings}
\alias{dbSetDataMappings}
\title{Set data mappings between an DBMS and R.}
\usage{
dbSetDataMappings(res, flds, ...)
}
\arguments{
\item{res}{An object inheriting from \link[=DBIResult-class]{DBI::DBIResult}.}

\item{flds}{a field description object as returned by \code{dbColumnInfo}.}

\item{...}{Other arguments passed on to methods.}
}
\description{
This generic is deprecated since no working implementation was ever produced.
}
\details{
Sets one or more conversion functions to handle the translation of DBMS data
types to R objects.  This is only needed for non-primitive data, since all
DBI drivers handle the common base types (integers, numeric, strings, etc.)

The details on conversion functions (e.g., arguments, whether they can invoke
initializers and/or destructors) have not been specified.
}
\keyword{internal}
