% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/06-ANSI.R
\name{ANSI}
\alias{ANSI}
\title{A dummy DBI connector that simulates ANSI-SQL compliance}
\usage{
ANSI()
}
\description{
A dummy DBI connector that simulates ANSI-SQL compliance
}
\examples{
ANSI()
}
\keyword{internal}
