% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dbCallProc.R
\name{dbCallProc}
\alias{dbCallProc}
\title{Call an SQL stored procedure}
\usage{
dbCallProc(conn, ...)
}
\arguments{
\item{conn}{A \link[=DBIConnection-class]{DBI::DBIConnection} object,
as returned by \code{\link[=dbConnect]{dbConnect()}}.}

\item{...}{Other parameters passed on to methods.}
}
\description{
\strong{Deprecated since 2014}
}
\details{
The recommended way of calling a stored procedure is now

\enumerate{
\item{\code{\link{dbGetQuery}} if a result set is returned}
\item{\code{\link{dbExecute}}  for data manipulation and other cases where no result set is returned}
}
}
\keyword{internal}
