% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/07-DBIResultArrow.R
\docType{class}
\name{DBIResultArrow-class}
\alias{DBIResultArrow-class}
\alias{DBIResultArrowDefault-class}
\title{DBIResultArrow class}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

This virtual class describes the result and state of execution of
a DBMS statement (any statement, query or non-query)
for returning data as an Arrow object.
}
\section{Implementation notes}{

Individual drivers are free to allow single or multiple
active results per connection.

The default show method displays a summary of the query using other
DBI generics.
}

\seealso{
Other DBI classes: 
\code{\link{DBIConnection-class}},
\code{\link{DBIConnector-class}},
\code{\link{DBIDriver-class}},
\code{\link{DBIObject-class}},
\code{\link{DBIResult-class}}

Other DBIResultArrow generics: 
\code{\link{dbBind}()},
\code{\link{dbClearResult}()},
\code{\link{dbFetchArrow}()},
\code{\link{dbFetchArrowChunk}()},
\code{\link{dbHasCompleted}()},
\code{\link{dbIsValid}()}
}
\concept{DBI classes}
\concept{DBIResultArrow generics}
